%%
% Generate random data set
rand('state',0);
randn('state',0);
x = bsxfun(@times,rand(50,1)*rand(1,50),shiftdim([0;.5;1],-2))+.5*rand(50,50,3);
x0 = x;

% Introduce missing data
x(rand(size(x))>.2) = nan;

% Components
K = 1;

% Display data
figure(1);
imagesc(x0);
title('Data');

%%
% Fit parafac model
OptPARAFAC(1) = 1e-32;
OptPARAFAC(2) = 10;
OptPARAFAC(5) = nan;
OptPARAFAC(6) = 2500;
F = parafac(x, K, OptPARAFAC,2*ones(ndims(x),1));
XH = nmodel(F);

% Display reconstruction
figure(2);
image(min(XH,1));
title('PARAFAC');


%% 
% Fit using Hamiltonian MCMC
[U,MU,V,VV] = ntfmcmc(x, K, 1000, 'method', 'hmcmc', 'vv', .01, 'epsilon', 1e-3, 'u', vertcat(F{:}));

% Average reconstruction over last 500 samples
rng = 500:1000;
XH = zeros(size(x));
for k = rng
    XH = XH+nmodel(mat2cell(U(:,:,k),size(x),K));
end
XH = XH./numel(rng);

% Display reconstruction
figure(3);
image(min(XH,1));
title('HMCMC');

% Display variance
figure(4);
plot(VV)
title('Variance');


